Class: ObjectForge::Forge::Parameters

Inherits:
Struct
  • Object
show all
Defined in:
lib/object_forge/forge.rb

Overview

Interface for forge parameters. It is not used internally, but can be useful for defining forges through means other than ObjectForge::ForgeDSL.

Since:

  • 0.1.0

Instance Attribute Summary collapse

Instance Attribute Details

#attributesHash{Symbol => Any} (readonly)

Non-trait values of the attributes.

Returns:

  • (Hash{Symbol => Any})


30
# File 'lib/object_forge/forge.rb', line 30

Parameters = Struct.new(:attributes, :traits, :settings, keyword_init: true)

#settingsHash{Symbol => Any} (readonly)

A forge’s settings. Must include a :mold key, containing an object that knows how to build the instance with a call method that takes a class and a hash of attributes.

Returns:

  • (Hash{Symbol => Any})

Since:

  • 0.3.0



30
# File 'lib/object_forge/forge.rb', line 30

Parameters = Struct.new(:attributes, :traits, :settings, keyword_init: true)

#traitsHash{Symbol => Hash{Symbol => Any}} (readonly)

Attributes belonging to traits.

Returns:

  • (Hash{Symbol => Hash{Symbol => Any}})


30
# File 'lib/object_forge/forge.rb', line 30

Parameters = Struct.new(:attributes, :traits, :settings, keyword_init: true)